FUNCTION CREFL_UNSCALE, DATA
;- Convert corrected reflectance scaled integer data to float and set invalid pixels to -1.0
loc = where(data eq 32767, count)
data = data * 0.0001
if (count gt 0) then data[loc] = -1.0
return, data
END

PRO MODIS_REPROJECT, QKM_PROJ_HDF, HKM_PROJ_HDF, OUT_TIFF, BANDS=BANDS, SHARPEN=SHARPEN

;+
; Create MODIS 250 meter RGB TIFF output given reprojected
; input files created by run_modis_reproject.bash
;
; Usage
; MODIS_REPROJECT, QKM_PROJ_HDF, HKM_PROJ_HDF, OUT_TIFF
;
; Input
; QKM_PROJ_HDF is the reprojected corrected reflectance (HDF) from QKM bands 1 and 2
; HKM_PROJ_HDF is the reprojected corrected reflectance (HDF) from HKM bands 1, 2, 3, 4
;
; Output
; OUT_TIFF is the output TIFF file name
;-

;- Check arguments
if (n_elements(bands) eq 0) then bands = [1, 4, 3]
if (n_elements(sharpen) eq 0) then sharpen = 1

;- Read reprojected QKM sharpening band
hdfid = hdf_sd_start(qkm_proj_hdf)
sds_name = string(sharpen, format='("CorrRefl_", i2.2)')
hdf_sd_varread, hdfid, sds_name, sharp_250
hdf_sd_end, hdfid

;- Read reprojected HKM sharpening band
hdfid = hdf_sd_start(hkm_proj_hdf)
hdf_sd_varread, hdfid, sds_name, sharp_500

;- Unscale the QKM and HKM sharpening bands
sharp_250 = crefl_unscale(temporary(sharp_250))
sharp_500 = crefl_unscale(temporary(sharp_500))

;- Compute sharpening ratio
ratio = temporary(sharp_500) / temporary(sharp_250)

;- Read reprojected HKM red, green, and blue bands from HDF
hdf_sd_varread, hdfid, string(bands[0], format='("CorrRefl_", i2.2)'), red
hdf_sd_varread, hdfid, string(bands[1], format='("CorrRefl_", i2.2)'), grn
hdf_sd_varread, hdfid, string(bands[2], format='("CorrRefl_", i2.2)'), blu

;- Close HKM HDF file
hdf_sd_end, hdfid

;- Convert to reflectance
red = crefl_unscale(temporary(red))
grn = crefl_unscale(temporary(grn))
blu = crefl_unscale(temporary(blu))

;- Create sharpened red, green, blue bands
loc = where(ratio gt 0.0, count)
if (count gt 0) then begin
  red[loc] = red[loc] / ratio[loc]
  grn[loc] = grn[loc] / ratio[loc]
  blu[loc] = blu[loc] / ratio[loc]
endif

;- Create RGB image
true = modis_false_color(temporary(red), temporary(grn), temporary(blu), /cloud)

;- Write TIFF true color image
write_tiff, out_tiff, true
print, out_tiff, format='("(Created TIFF image ", a, ")")'

END
